#include "zpuino.h"
#include <stdarg.h>
#include <string.h>

//#undef DEBUG_SERIAL
//#define SIMULATION
//#define VERBOSE_LOADER
//#define BOOT_IMMEDIATLY

#define BOOTLOADER_SIZE 0x1000

#ifdef SIMULATION
# define SPICODESIZE 0x1000
# define FPGA_SS_B 40
# undef SPI_FLASH_SEL_PIN
# define SPI_FLASH_SEL_PIN FPGA_SS_B
#else
# define SPICODESIZE (BOARD_MEMORYSIZE - BOOTLOADER_SIZE - 128)
#endif
#define VERSION_HIGH 0x01
#define VERSION_LOW  0x09

/* Commands for programmer */

#define BOOTLOADER_CMD_VERSION 0x01
#define BOOTLOADER_CMD_IDENTIFY 0x02
#define BOOTLOADER_CMD_WAITREADY 0x03
#define BOOTLOADER_CMD_RAWREADWRITE 0x04
#define BOOTLOADER_CMD_ENTERPGM 0x05
#define BOOTLOADER_CMD_LEAVEPGM 0x06
#define BOOTLOADER_CMD_SSTAAIPROGRAM 0x07
#define BOOTLOADER_CMD_SETBAUDRATE 0x08
#define BOOTLOADER_CMD_PROGMEM 0x09
#define BOOTLOADER_CMD_START 0x0A
#define BOOTLOADER_MAX_CMD 0x0A

#ifdef SIMULATION
# define BOOTLOADER_WAIT_MILLIS 10
#else
# define BOOTLOADER_WAIT_MILLIS 1000
#endif

#define REPLY(X) (X|0x80)

#define HDLC_frameFlag 0x7E
#define HDLC_escapeFlag 0x7D
#define HDLC_escapeXOR 0x20

#define BDATA /*__attribute__((section(".bdata")))*/

extern "C" void (*ivector)(void);
extern "C" void *bootloaderdata;

static BDATA int inprogrammode;
static BDATA volatile unsigned int milisseconds;
static BDATA unsigned int flash_id;

struct bootloader_data_t {
	unsigned int spiend;
	unsigned int signature;
	const unsigned char *vstring;
};

struct bootloader_data_t bdata BDATA;

const unsigned char vstring[] = {
	VERSION_HIGH,
	VERSION_LOW,
	SPIOFFSET>>16,
	SPIOFFSET>>8,
	SPIOFFSET&0xff,
	SPICODESIZE>>16,
	SPICODESIZE>>8,
	SPICODESIZE&0xff,
	CLK_FREQ >> 24,
	CLK_FREQ >> 16,
	CLK_FREQ >> 8,
	CLK_FREQ,
	BOARD_ID >> 24,
	BOARD_ID >> 16,
	BOARD_ID >> 8,
	BOARD_ID
};


static void outbyte(int);

void flush()
{
	/* Flush serial line */
	while (UARTSTATUS & 4);
}

extern "C" void printnibble(unsigned int c)
{
	c&=0xf;
	if (c>9)
		outbyte(c+'a'-10);
	else
		outbyte(c+'0');
}
extern "C" void printstring(const char *str)
{
	while (*str) {
		outbyte(*str);
		str++;
	}
}

extern "C" void printhexbyte(unsigned int c)
{
	printnibble(c>>4);
	printnibble(c);
}
extern "C" void printhex(unsigned int c)
{
	printhexbyte(c>>24);
	printhexbyte(c>>16);
	printhexbyte(c>>8);
	printhexbyte(c);
}

extern "C" void spi_copy() __attribute__((noreturn));
extern "C" void start_sketch() __attribute__((noreturn));

#ifdef DEBUG_SERIAL
const unsigned char serialbuffer[] = {
	HDLC_frameFlag, BOOTLOADER_CMD_RAWREADWRITE, 0x5, 0x10, 0xB, 0x0, 0x0, 0x0, 0x0, /*0x9F*/0x55, 0xAD, HDLC_frameFlag,
	HDLC_frameFlag, BOOTLOADER_CMD_IDENTIFY, 0x2c, 0x95, HDLC_frameFlag,

};
int serialbufferptr=0;
#endif

void sendByte(unsigned int i)
{
	CRC16APP = i;
	i &= 0xff;
	if (i==HDLC_frameFlag || i==HDLC_escapeFlag) {
		outbyte(HDLC_escapeFlag);
		outbyte(i ^ HDLC_escapeXOR);
	} else
		outbyte(i);
}

static void sendBuffer(const unsigned char *buf, unsigned int size)
{
	while (size--!=0)
		sendByte(*buf++);
}

static void prepareSend()
{
	CRC16ACC=0xFFFF;
	outbyte(HDLC_frameFlag);
}


void finishSend()
{
	unsigned int crc = CRC16ACC;
	sendByte(crc>>8);
	sendByte(crc&0xff);
	outbyte(HDLC_frameFlag);
}

static unsigned int inbyte()
{
#ifdef BOOT_IMMEDIATLY
		spi_copy();
#else

	for (;;)
	{
#ifdef DEBUG_SERIAL
		if (serialbufferptr<sizeof(serialbuffer))
			return serialbuffer[serialbufferptr++];
#else

		if (UARTCTL&0x1 != 0) {
			return UARTDATA;
		}

#endif

		if (inprogrammode==0 && milisseconds>BOOTLOADER_WAIT_MILLIS) {
			INTRCTL=0;
			TMR0CTL=0;
#ifdef __ZPUINO_NEXYS3__
			digitalWrite(FPGA_LED_0, LOW);
#endif
			spi_copy();
		}
	}
#endif
}

static void enableTimer()
{
#ifdef BOOT_IMMEDIATLY
	return; // TEST
#endif

#ifdef SIMULATION
	TMR0CMP = (CLK_FREQ/100000U)-1;
#else
	TMR0CMP = (CLK_FREQ/2000U)-1;
#endif
	TMR0CNT = 0x0;
	TMR0CTL = BIT(TCTLENA)|BIT(TCTLCCM)|BIT(TCTLDIR)|BIT(TCTLCP0)|BIT(TCTLIEN);
}



/*
 * Output one character to the serial port 
 * 
 * 
 */
static void outbyte(int c)
{
	/* Wait for space in FIFO */
	while ((UARTCTL&0x2)==2);
	UARTDATA=c;
}

static void spi_disable(register_t base)
{
	(void)*base; // Let SPI finish
	digitalWrite(SPI_FLASH_SEL_PIN,HIGH);
}

static void spi_enable()
{
	digitalWrite(SPI_FLASH_SEL_PIN,LOW);
}

static void spi_reset(register_t base)
{
	spi_disable(base);
	spi_enable();
	spi_disable(base);
}

static inline void waitspiready()
{
#if 0
	while (!(SPICTL & BIT(SPIREADY)));
#endif
}

static inline void spiwrite(unsigned int i)
{
	waitspiready();
	SPIDATA=i;
}

static inline unsigned int spiread()
{
	waitspiready();
	return SPIDATA;
}

static inline void spiwrite(register_t base, unsigned int i)
{
	waitspiready();
	*base=i;
}

static inline unsigned int spiread(register_t base)
{
	waitspiready();
	return *base;
}

extern "C" void __attribute__((noreturn)) start()
{
	ivector = (void (*)(void))0x1010;
	bootloaderdata = &bdata;
	start_sketch();
}

static unsigned start_read_size(register_t spidata)
{
	spiwrite(spidata,0x0B);
	spiwrite(spidata+4,SPIOFFSET);
	spiwrite(spidata+4,0);
	return spiread(spidata) & 0xffff;
}

extern "C" void copy_sketch(register_t spidata, unsigned crc16base, unsigned sketchsize, volatile unsigned *target)
{
	while (sketchsize--) {
		for (int i=4;i!=0;i--) {
			spiwrite(spidata,0);
			REGISTER(crc16base,ROFF_CRC16APP)=spiread(spidata);
		}
		*target++ = spiread(spidata);
	}
}

extern "C" void __attribute__((noreturn)) spi_copy_impl()
{
	// We must not overflow stack, leave 128 bytes
	//unsigned int count = SPICODESIZE >> 2; // 0x7000
	volatile unsigned int *board = (volatile unsigned int*)0x1004;
	volatile unsigned int *target = (volatile unsigned int *)0x1000;
	register_t spidata = &SPIDATA; // Ensure this stays in stack
	unsigned int sketchsize;
	unsigned int sketchcrc;
	unsigned crc16base = CRC16BASE;

#ifdef VERBOSE_LOADER
	printstring("CP\r\n");
#endif
#ifdef __ZPUINO_NEXYS3__
	digitalWrite(FPGA_LED_1, HIGH);
#endif


	spi_enable();
	sketchsize=start_read_size(spidata);
	bdata.spiend = (sketchsize<<2) + SPIOFFSET + 4;
	bdata.signature = 0xb00110ad;
	bdata.vstring=vstring;
	spiwrite(spidata,0);
	spiwrite(spidata,0);
	sketchcrc= spiread(spidata) & 0xffff;

	if (sketchsize>SPICODESIZE) {
#ifdef VERBOSE_LOADER
		printstring("SLK");
		//printhexbyte((sketchsize>>8)&0xff);
		//printhexbyte((sketchsize)&0xff);
		printstring("\r\n");
#endif
#ifdef __ZPUINO_NEXYS3__
		digitalWrite(FPGA_LED_2, HIGH);
#endif

		while(1) {}
	}

	//CRC16ACC=0xFFFF;
	REGISTER(crc16base,ROFF_CRC16ACC) = 0xffff;

#ifdef VERBOSE_LOADER
	//printstring("Filling\n");
#endif
    copy_sketch(spidata, crc16base, sketchsize, target);
#ifdef VERBOSE_LOADER
   // printstring("Filled\n");
#endif

	spi_disable(spidata);

	if (sketchcrc != REGISTER(crc16base,ROFF_CRC16ACC)) {
		outbyte('C');
        //printstring("CRC");
//		printstring("CRC error, please reset\r\n");
		/*
		printhex(sketchcrc);
		printstring(" ");
		printhex(CRC16ACC);
		printstring("\r\n");
		*/
#ifdef __ZPUINO_NEXYS3__
		digitalWrite(FPGA_LED_3, HIGH);
#endif

		while(1) {};
	}

	if (*board != BOARD_ID) {
        outbyte('B');
		//printstring("B!");
		//printhex(*board);
		//printstring(" != ");
		//printhex(BOARD_ID);
#ifdef __ZPUINO_NEXYS3__
		digitalWrite(FPGA_LED_4, HIGH);
#endif

		while(1) {};
	}

#ifdef VERBOSE_LOADER
	printstring("Loaded, starting...\r\n");
#endif
	SPICTL &= ~(BIT(SPIEN));

	// Reset settings
	/*
	 GPIOTRIS(0) = 0xffffffff;
	 GPIOTRIS(1) = 0xffffffff;
	 GPIOTRIS(2) = 0xffffffff;
	 GPIOTRIS(3) = 0xffffffff;
	 */
	flush();
	start();
	//asm ("im _start\npoppc\nnop\n");
	while (1) {}
}


extern "C" void _zpu_interrupt()
{
	milisseconds++;
//	outbyte('I');
	TMR0CTL &= ~(BIT(TCTLIF));
}

static inline int is_atmel_flash()
{
	//return ((flash_id & 0xff0000)==0x1f0000);
	return 0;
}

static void simpleReply(unsigned int r)
{
	prepareSend();
	sendByte(REPLY(r));
	finishSend();
}

static int spi_read_status()
{
	unsigned int status;
	register_t spidata = &SPIDATA; // Ensure this stays in stack

	spi_enable();
#if 0
	if (is_atmel_flash())
		spiwrite(spidata,0x57);
	else
#endif
		spiwrite(spidata,0x05);

	spiwrite(spidata,0x00);
	status =  spiread(spidata) & 0xff;
	spi_disable(spidata);
	return status;
}

static unsigned int spi_read_id()
{
	unsigned int ret;
	register_t spidata = &SPIDATA; // Ensure this stays in stack

	spi_enable();
    /*
	spiwrite(spidata,0x9F);
	spiwrite(spidata,0x00);
	spiwrite(spidata,0x00);
	spiwrite(spidata,0x00);
	*/
	spiwrite(spidata+6, 0x9f000000);
	ret = spiread(spidata);
	spi_disable(spidata);
	return ret;
}

static void cmd_progmem(unsigned char *buffer)
{
	/* Directly program memory. */

	/*
	 buffer[1-4] is address.
	 buffer[5] is size,
	 next bytes are data
	 */

	unsigned int address, size=5;
	volatile unsigned char *mem;
	unsigned char *source;

	address=(buffer[1]<<24);
	address+=(buffer[2]<<16);
	address+=(buffer[3]<<8);
	address+=buffer[4];
	mem = (volatile unsigned char *)address + 0x1000;

	size=buffer[5];
	source = &buffer[6];

	while (size--) {
		*mem++=*source++;
	}
	simpleReply(BOOTLOADER_CMD_PROGMEM);

}


static void cmd_raw_send_receive(unsigned char *buffer)
{
	unsigned int count;
	unsigned int rxcount;
	unsigned int txcount;
	register_t spidata = &SPIDATA; // Ensure this stays in stack

	// buffer[1-2] is number of TX bytes
	// buffer[3-4] is number of RX bytes
	// buffer[5..] is data to transmit.

	// NOTE - buffer will be overwritten in read.

	spi_enable();
	txcount = buffer[1];
	txcount<<=8;
	txcount += buffer[2];

	for (count=0; count<txcount; count++) {
		spiwrite(spidata,buffer[5+count]);
	}
	rxcount = buffer[3];
	rxcount<<=8;
	rxcount += buffer[4];
	// Now, receive and write buffer
	for(count=0;count <rxcount;count++) {
		spiwrite(spidata,0x00);
		buffer[count] = spiread(spidata);
	}
	spi_disable(spidata);

	// Send back
	prepareSend();
	sendByte(REPLY(BOOTLOADER_CMD_RAWREADWRITE));
	sendByte(rxcount>>8);
	sendByte(rxcount);
	for(count=0;count<rxcount;count++) {
		sendByte(buffer[count]);
	}
	finishSend();
}


static void cmd_sst_aai_program(unsigned char *buffer)
{
	unsigned int count;
	unsigned int txcount;
	register_t spidata = &SPIDATA; // Ensure this stays in stack

#ifdef __SST_FLASH__

	// buffer[1-2] is number of TX bytes
    // buffer[3-5] is address to program
	// buffer[6...] is data to transmit.

	// Enable writes
	spi_enable();
	spiwrite(spidata,0x06);
	spi_disable(spidata);

	spi_enable();
	spiwrite(spidata,0xAD);

	txcount = buffer[1];
	txcount<<=8;
	txcount += buffer[2];

	spiwrite(spidata,buffer[3]);
	spiwrite(spidata,buffer[4]);
	spiwrite(spidata,buffer[5]);

	for (count=0; count<txcount; count+=2) {
		if (count>0) {
			spi_enable();
			spiwrite(spidata,0xAD);
		}
		spiwrite(spidata,buffer[6+count]);
		spiwrite(spidata,buffer[6+count+1]);
		spi_disable(spidata);
		// Read back status, wait for completion
		while (spi_read_status() & 1);
	}

	// Disable write enable

	spi_enable();
	spiwrite(spidata,0x04);
	spi_disable(spidata);
	// Send back
	prepareSend();
	sendByte(REPLY(BOOTLOADER_CMD_SSTAAIPROGRAM));
	finishSend();
#endif

}

static void cmd_set_baudrate(unsigned char *buffer)
{

	unsigned int bsel = buffer[1];
	bsel<<=8;
	bsel |= buffer[2];
	bsel<<=8;
	bsel |= buffer[3];
	bsel<<=8;
    bsel |= buffer[4];

	simpleReply(BOOTLOADER_CMD_SETBAUDRATE);

	// We ought to wait here, to ensure output is properly drained.
	outbyte(0xff);
	while ((UARTCTL&0x2)==2);

	UARTCTL = bsel | BIT(UARTEN);
}


static void cmd_waitready(unsigned char *buffer)
{
	int status;

	if (is_atmel_flash()) {
		do {
			status = spi_read_status();
		} while (!(status & 0x80));
	} else {
		do {
			status = spi_read_status();
		} while (status & 1);
	}
	prepareSend();
	sendByte(REPLY(BOOTLOADER_CMD_WAITREADY));
	sendByte(status);
	finishSend();
}


static void cmd_version(unsigned char *buffer)
{
	// Reset boot counter
	milisseconds = 0;
	prepareSend();
	sendByte(REPLY(BOOTLOADER_CMD_VERSION));

	sendBuffer(vstring,sizeof(vstring));
	finishSend();
}

static void cmd_identify(unsigned char *buffer)
{
	// Reset boot counter
	milisseconds = 0;
	int id;

	prepareSend();
	sendByte(REPLY(BOOTLOADER_CMD_IDENTIFY));
	flash_id = spi_read_id();
	sendByte(flash_id>>16);
	sendByte(flash_id>>8);
	sendByte(flash_id);
	id = spi_read_status();
	sendByte(id);
	finishSend();
}


static void cmd_enterpgm(unsigned char *buffer)
{
	inprogrammode = 1;
	// Disable timer.
	TMR0CTL = 0;
	simpleReply(BOOTLOADER_CMD_ENTERPGM);
}

static void cmd_leavepgm(unsigned char *buffer)
{
	inprogrammode = 0;

	enableTimer();
	simpleReply(BOOTLOADER_CMD_LEAVEPGM);
}
 

void cmd_start(unsigned char *buffer)
{
	register_t spidata = &SPIDATA; // Ensure this stays in stack
	simpleReply(BOOTLOADER_CMD_START);

	// Make sure we keep at least smallFS data

	spi_enable();
	bdata.spiend = (start_read_size(spidata)<<2) + SPIOFFSET + 4;
	bdata.signature = 0xb00110ad;
	bdata.vstring=vstring;
	spi_disable(spidata);
	flush();
	start();
}

typedef void(*cmdhandler_t)(unsigned char *);

static const cmdhandler_t handlers[] = {
	&cmd_version,         /* CMD1 */
	&cmd_identify,        /* CMD2 */
	&cmd_waitready,       /* CMD3 */
	&cmd_raw_send_receive,/* CMD4 */
	&cmd_enterpgm,        /* CMD5 */
	&cmd_leavepgm,        /* CMD6 */
	&cmd_sst_aai_program, /* CMD7 */
	&cmd_set_baudrate,    /* CMD8 */
	&cmd_progmem,         /* CMD9 */
	&cmd_start            /* CMD10 */
};


inline void processCommand(unsigned char *buffer, unsigned bufferpos)
{
	unsigned int pos=0;
	if (bufferpos<3)
		return; // Too few data

	CRC16ACC=0xFFFF;
	for (pos=0;pos<bufferpos-2;pos++) {
		CRC16APP=buffer[pos];
	}
	unsigned int tcrc = buffer[--bufferpos];
	tcrc|=buffer[--bufferpos]<<8;
	unsigned int rcrc=CRC16ACC;
	if (rcrc!=tcrc) {
		//printstring("C!");
		return;
	}

	pos=buffer[0];

	if (pos>BOOTLOADER_MAX_CMD)
		return;
	pos--;
	handlers[pos](buffer);
}

#ifdef __ZPUINO_S3E_EVAL__

inline void configure_pins()
{
	digitalWrite(FPGA_AD_CONV,LOW);
	digitalWrite(FPGA_DAC_CS,HIGH);
	digitalWrite(FPGA_AMP_CS,HIGH);
	digitalWrite(SPI_FLASH_SEL_PIN,HIGH);

	pinMode(SPI_FLASH_SEL_PIN, OUTPUT);
	pinMode(FPGA_AD_CONV, OUTPUT);
	pinMode(FPGA_DAC_CS, OUTPUT);
	pinMode(FPGA_AMP_CS, OUTPUT);

	pinMode(FPGA_LED_0, OUTPUT);

	digitalWrite(FPGA_LED_0, HIGH);
}
#endif

#ifdef __ZPUINO_PAPILIO_ONE__
inline void configure_pins()
{
	pinModePPS(FPGA_PIN_FLASHCS,LOW);
	pinMode(FPGA_PIN_FLASHCS, OUTPUT);
}
#endif

//add yoko 2015/02/02
#ifdef __ZPUINO_PAPILIO_ONE_MB__
inline void configure_pins()
{
	pinModePPS(FPGA_PIN_FLASHCS,LOW);
	pinMode(FPGA_PIN_FLASHCS, OUTPUT);
}
#endif

#ifdef __ZPUINO_OHO_GODIL__
inline void configure_pins()
{
	pinModePPS(FPGA_PIN_FLASHCS,LOW);
	pinMode(FPGA_PIN_FLASHCS, OUTPUT);
}
#endif

#ifdef __ZPUINO_XULA2__
inline void configure_pins()
{
	pinModePPS(FPGA_PIN_FLASHCS,LOW);
	pinMode(FPGA_PIN_FLASHCS, OUTPUT);
	pinModePPS(FPGA_PIN_SDCS,LOW);
        pinMode(FPGA_PIN_SDCS, OUTPUT);
        digitalWrite(FPGA_PIN_SDCS, HIGH);
}
#endif

#if defined( __ZPUINO_PAPILIO_PLUS__ ) || defined( __ZPUINO_PAPILIO_PRO__ ) || defined ( __ZPUINO_PAPILIO_DUO__ )
inline void configure_pins()
{
	pinModePPS(FPGA_PIN_FLASHCS,LOW);
	pinMode(FPGA_PIN_FLASHCS, OUTPUT);
}
#endif
#ifdef __ZPUINO_NEXYS2__
inline void configure_pins()
{
	pinModePPS(FPGA_PMOD_JA_2,LOW);
	pinMode(FPGA_PMOD_JA_2, OUTPUT);
	digitalWrite(FPGA_PMOD_JA_2,HIGH);
}
#endif
#ifdef __ZPUINO_NEXYS3__
inline void configure_pins()
{
	digitalWrite(SPI_FLASH_SEL_PIN,HIGH);
	digitalWrite(FPGA_LED_0,HIGH);
}
#endif

extern "C" int _syscall(int *foo, int ID, ...);
extern "C" unsigned _bfunctions[];

extern "C" void udivmodsi4(); /* Just need it's address */

extern "C" int loadsketch(unsigned offset, unsigned size)
{
	register_t spidata = &SPIDATA; // Ensure this stays in stack
	unsigned crc16base = CRC16BASE;
	volatile unsigned int *target = (volatile unsigned int *)0x1000;
	spi_disable(spidata);
	spi_enable();
	spiwrite(spidata,0x0b);
	spiwrite(spidata+4,offset);
	spiwrite(spidata,0x0);
	copy_sketch(spidata, crc16base, size, target);
	spi_disable(spidata);
	flush();
	start();
}

extern "C" int main(int argc,char**argv)
{
	inprogrammode = 0;
	milisseconds = 0;
	unsigned bufferpos = 0;
	unsigned char buffer[256 + 32];
	int syncSeen;
	int unescaping;

	ivector = &_zpu_interrupt;

	UARTCTL = BAUDRATEGEN(115200) | BIT(UARTEN);

	configure_pins();

#ifndef VERBOSE_LOADER
	_bfunctions[0] = (unsigned)&udivmodsi4;
	_bfunctions[1] = (unsigned)&memcpy;
	_bfunctions[2] = (unsigned)&memset;
	_bfunctions[3] = (unsigned)&strcmp;
	_bfunctions[4] = (unsigned)&loadsketch;
#endif

	INTRMASK = BIT(INTRLINE_TIMER0); // Enable Timer0 interrupt
	INTRCTL=1;

#ifdef VERBOSE_LOADER
	printstring("\r\nZPUINO\r\n");
#endif


	enableTimer();

	CRC16POLY = 0x8408; // CRC16-CCITT
	SPICTL=BIT(SPICPOL)|BOARD_SPI_DIVIDER|BIT(SPISRE)|BIT(SPIEN)|BIT(SPIBLOCK);
	// Reset flash
	spi_reset(&SPIDATA);
#ifdef __SST_FLASH__
	spi_enable();
	spiwrite(0x4); // Disable WREN for SST flash
	spi_disable(&SPIDATA);
#endif

	syncSeen = 0;
	unescaping = 0;
	while (1) {
		int i;
		i = inbyte();
		// DEBUG ONLY
		//TMR1CNT=i;
		//outbyte(i);
		if (syncSeen) {
			if (i==HDLC_frameFlag) {
				if (bufferpos>0) {
					syncSeen=0;
					processCommand(buffer, bufferpos);
				}
			} else if (i==HDLC_escapeFlag) {
				unescaping=1;
			} else if (bufferpos<sizeof(buffer)) {
				if (unescaping) {
					unescaping=0;
					i^=HDLC_escapeXOR;
				}
				buffer[bufferpos++]=i;
			} else {
				syncSeen=0;
			}
		} else {
			if (i==HDLC_frameFlag) {
				bufferpos=0;
				CRC16ACC=0xFFFF;
				syncSeen=1;
				unescaping=0;
			} else {
#ifdef VERBOSE_LOADER
				//outbyte(i); // Echo back.
#endif
			}
		}
	}
}
